/***
*
* BNBT Beta 8.0 - A C++ BitTorrent Tracker
* Copyright (C) 2003-2004 Trevor Hogan
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
***/

// =Xotic= Modified Source File

#include "bnbt.h"
#include "bnbt_mysql.h"
#include "atom.h"
#include "bencode.h"
#include "tracker.h"
#include "util.h"	   

// Added by =Xotic=
#include "link.h"  
#include "xconfig_lang.h"
// --------------------- End of Addition

void CTracker :: serverResponseAnnounce( struct request_t *pRequest, struct response_t *pResponse )
{
	pResponse->strCode = "200 " + gmapLANG_CFG["server_response_200"];

	pResponse->mapHeaders.insert( pair<string, string>( "Content-Type", "text/plain" ) );

	// retrieve info hash

	string strInfoHash = pRequest->mapParams["info_hash"];

	if( strInfoHash.empty( ) )
	{
		pResponse->strContent = UTIL_FailureReason( gmapLANG_CFG["announce_hash_missing"] );
		pResponse->bCompressOK = false;

		return;
	}

	if( m_pAllowed )
	{
		if( !m_pAllowed->getItem( strInfoHash ) )
		{
			pResponse->strContent = UTIL_FailureReason( gmapLANG_CFG["announce_download_not_authorized"] );
			pResponse->bCompressOK = false;

			return;
		}
	}

	// retrieve ip

	string strIP = inet_ntoa( pRequest->sin.sin_addr );
	string strTempIP = pRequest->mapParams["ip"];
	
	// Added and modified buy =Xotic=
	// CBTT
	string strIPConv = strIP.c_str( );

	// Verify that the IP is permitted to access the tracker
	if( ( isIPBanList( strIP.c_str( ) ) == 1 || isIPBanList( strTempIP ) == 1 ) && m_iIPBanMode == 1 )
	{
		pResponse->strContent = UTIL_FailureReason( gmapLANG_CFG["announce_ip_banned"] );
		pResponse->bCompressOK = false;
		m_tXStats.announce.iIPBanned++;
		return;
	}
	else if( ( isIPBanList( strIP.c_str( ) ) == 0 || isIPBanList( strTempIP ) == 0 ) && m_iIPBanMode == 2 )
	{
		pResponse->strContent = UTIL_FailureReason( gmapLANG_CFG["announce_ip_not_cleared"] );
		pResponse->bCompressOK = false;
		m_tXStats.announce.iIPNotCleared++;
		return;
	}

	if( m_iBlockNATedIP == 1 && ( !strTempIP.empty( ) && ( strTempIP.substr(0,8) == "192.168." || strTempIP.substr(0,8) == "169.254." || strTempIP.substr(0,3) == "10." || strTempIP.substr(0,7) == "172.16." || strTempIP.substr(0,7) == "172.17." || strTempIP.substr(0,7) == "172.18." || strTempIP.substr(0,7) == "172.19." || strTempIP.substr(0,7) == "172.20." || strTempIP.substr(0,7) == "172.21." || strTempIP.substr(0,7) == "172.22." || strTempIP.substr(0,7) == "172.23." || strTempIP.substr(0,7) == "172.24." || strTempIP.substr(0,7) == "172.25." || strTempIP.substr(0,7) == "172.26." || strTempIP.substr(0,7) == "172.27." || strTempIP.substr(0,7) == "172.28." || strTempIP.substr(0,7) == "172.29." || strTempIP.substr(0,7) == "172.30." || strTempIP.substr(0,7) == "172.31." || strTempIP.substr(0,7) == "172.32." || strTempIP == "127.0.0.1" ) ) )
	{
		strTempIP = string( );
	}

	if( m_iLocalOnly == 1 && ( !strIPConv.empty( ) && ( strIPConv.substr(0,8) == "192.168." || strIPConv.substr(0,8) == "169.254." || strIPConv.substr(0,3) == "10." || strIPConv.substr(0,7) == "172.16." || strIPConv.substr(0,7) == "172.17." || strIPConv.substr(0,7) == "172.18." || strIPConv.substr(0,7) == "172.19." || strIPConv.substr(0,7) == "172.20." || strIPConv.substr(0,7) == "172.21." || strIPConv.substr(0,7) == "172.22." || strIPConv.substr(0,7) == "172.23." || strIPConv.substr(0,7) == "172.24." || strIPConv.substr(0,7) == "172.25." || strIPConv.substr(0,7) == "172.26." || strIPConv.substr(0,7) == "172.27." || strIPConv.substr(0,7) == "172.28." || strIPConv.substr(0,7) == "172.29." || strIPConv.substr(0,7) == "172.30." || strIPConv.substr(0,7) == "172.31." || strIPConv.substr(0,7) == "172.32." || strIPConv == "127.0.0.1" ) ) )
	{
		if( !strTempIP.empty( ) && strTempIP.find_first_not_of( "1234567890." ) == string :: npos )
		{
			strIP = strTempIP;
		}
	}
	else if( m_iLocalOnly == 0 )
	{
		if( !strTempIP.empty( ) && strTempIP.find_first_not_of( "1234567890." ) == string :: npos )
		{		
			strIP = strTempIP;
		}
	}
	// ----------------------------------------------------- End Modification

	// retrieve a ton of parameters

	string strEvent = pRequest->mapParams["event"];
	string strPort = pRequest->mapParams["port"];
	string strUploaded = pRequest->mapParams["uploaded"];
	string strDownloaded = pRequest->mapParams["downloaded"];
	string strLeft = pRequest->mapParams["left"];
	string strPeerID = pRequest->mapParams["peer_id"];
	string strNumWant = pRequest->mapParams["num_want"];
	string strNoPeerID = pRequest->mapParams["no_peer_id"];
	string strCompact = pRequest->mapParams["compact"];
	
	// Added by =Xotic=
	// CBTT
	string strUserAgent = pRequest->mapHeaders["User-Agent"];
    string struAgent = pRequest->mapHeaders["User-agent"];
	if( strUserAgent.empty( ) )
		strUserAgent = gmapLANG_CFG["unknown"];

	if( struAgent.empty( ) )
		struAgent = gmapLANG_CFG["unknown"];

	if( i_intPeerSpoofRestrict == 1 )
		if( ( "-AZ" != strPeerID.substr(0,3) && "Azureus" == strUserAgent.substr(0,7) ) || ( "BitTorrent/3." == strUserAgent.substr(0,13) && ( "S5" == strPeerID.substr(0,2) || UTIL_EscapedToString("S%05") == strPeerID.substr(0,2) ) ) )
		{
			pResponse->strCode = "200 " + gmapLANG_CFG["server_response_200"];
			pResponse->strContent = UTIL_FailureReason( gmapLANG_CFG["announce_client_spoof"] );
			m_tXStats.announce.iPeerSpoofRestrict++;
			return;
		}

	int isbanned = isClientBanList( strUserAgent , true );

	if( isbanned == 0 )
		isbanned = isClientBanList( struAgent , true );
	if( isbanned == 0 )
		isbanned = isClientBanList( strPeerID );

	if (isbanned == 1 && i_intBanMode == 1)
	{
		pResponse->strCode = "200 " + gmapLANG_CFG["server_response_200"];
		pResponse->strContent = UTIL_FailureReason( gmapLANG_CFG["announce_client_banned"] );
//		UTIL_LogPrint("Banned client denied access\n");
		m_tXStats.announce.iClientBanned++;
		return;
	}

	if (isbanned == 0 && i_intBanMode == 2)
	{
		pResponse->strCode = "200 " + gmapLANG_CFG["server_response_200"];
		pResponse->strContent = UTIL_FailureReason( gmapLANG_CFG["announce_client_banned"] );
//		UTIL_LogPrint("Non-Friendly Client denied access\n");
		m_tXStats.announce.iClientBanned++;
		return;
	}
	// ------------------------------------------------------- End Addition

	if( !strEvent.empty( ) && strEvent != "started" && strEvent != "completed" && strEvent != "stopped" )
	{
		pResponse->strContent = UTIL_FailureReason( gmapLANG_CFG["announce_invalid_event"] );
		pResponse->bCompressOK = false;
		m_tXStats.announce.iInvalidEvent++;

		return;
	}

	if( strPort.empty( ) )
	{
		pResponse->strContent = UTIL_FailureReason( gmapLANG_CFG["announce_port_missing"] );
		pResponse->bCompressOK = false;
		m_tXStats.announce.iPortMissing++;

		return;
	}

	if( strUploaded.empty( ) )
	{
		pResponse->strContent = UTIL_FailureReason( gmapLANG_CFG["announce_uploaded_missing"] );
		pResponse->bCompressOK = false;
		m_tXStats.announce.iUploadedMissing++;

		return;
	}

	if( strDownloaded.empty( ) )
	{
		pResponse->strContent = UTIL_FailureReason( gmapLANG_CFG["announce_downloaded_missing"] );
		pResponse->bCompressOK = false;
		m_tXStats.announce.iDownloadedMissing++;

		return;
	}

	if( strLeft.empty( ) )
	{
		pResponse->strContent = UTIL_FailureReason( gmapLANG_CFG["announce_left_missing"] );
		pResponse->bCompressOK = false;
		m_tXStats.announce.iLeftMissing++;

		return;
	}

	if( strPeerID.size( ) != 20 )
	{
		pResponse->strContent = UTIL_FailureReason( gmapLANG_CFG["announce_peerid_length"] );
		pResponse->bCompressOK = false;
		m_tXStats.announce.iPeerIDLength++;

		return;
	}

	struct announce_t ann;

	ann.strInfoHash = strInfoHash;
	ann.strIP = strIP;
	ann.strEvent = strEvent;
	ann.iPort = (unsigned int)atoi( strPort.c_str( ) );
	ann.iUploaded = UTIL_StringTo64( strUploaded.c_str( ) );
	ann.iDownloaded = UTIL_StringTo64( strDownloaded.c_str( ) );
	ann.iLeft = UTIL_StringTo64( strLeft.c_str( ) );
	ann.strPeerID = strPeerID;

	// Added by =Xotic
	// CBTT
	if( ( ( ann.iPort >= 6881 && ann.iPort <= 6999 ) || ( ann.iPort >= 411 && ann.iPort <= 413 ) || ann.iPort == 1214 || ann.iPort == 4662 || ( ann.iPort >= 6346 && ann.iPort <= 6347 ) )&& m_iBlacklistP2PPorts )
	{
		// Listen port is blacklisted
		pResponse->strCode = "200 " + gmapLANG_CFG["server_response_200"];
		pResponse->strContent = UTIL_FailureReason( UTIL_Xsprintf( gmapLANG_CFG["announce_blacklisted_port"].c_str( ), strPort.c_str( ) ) );
		m_tXStats.announce.iPortBlacklisted++;
		
		return;
	}

	int64 iOverFlowLimit = UTIL_StringTo64( strOverFlowLimit.c_str() );
	int64 iOverFlowminimum = UTIL_StringTo64( "107374182400" );
	if( iOverFlowLimit < iOverFlowminimum )
		iOverFlowLimit = iOverFlowminimum;

	if( ann.iDownloaded < 0 || ( m_iRestrictOverflow == 1 && ann.iDownloaded >= iOverFlowLimit ) )
	{
		// Invalid downloaded value reported
		pResponse->strCode = "200 " + gmapLANG_CFG["server_response_200"];
		pResponse->strContent = UTIL_FailureReason( gmapLANG_CFG["announce_downloaded_invalid"] );
		m_tXStats.announce.iDownloadedInvalid++;
		
		return;
	}

	if( ann.iUploaded < 0 || ( m_iRestrictOverflow == 1 && ann.iUploaded >= iOverFlowLimit ) )
	{
		// Invalid uploaded value reported
		pResponse->strCode = "200 " + gmapLANG_CFG["server_response_200"];
		pResponse->strContent = UTIL_FailureReason( gmapLANG_CFG["announce_uploaded_invalid"] );
		m_tXStats.announce.iUploadedInvalid++;

		return;
	}
	//---------------- End of Addition

	Announce( ann );

	unsigned int iRSize = m_iResponseSize;

	if( !strNumWant.empty( ) )
		iRSize = atoi( strNumWant.c_str( ) );

	if( iRSize > (unsigned int)m_iMaxGive )
		iRSize = (unsigned int)m_iMaxGive;

	// Added by =Xotic
	// link

	if( gpLinkServer || gpLink )
	{
		CAtomDicti *pAnnounce = new CAtomDicti( );

		pAnnounce->setItem( "info_hash", new CAtomString( ann.strInfoHash ) );
		pAnnounce->setItem( "ip", new CAtomString( ann.strIP ) );

		if( !strEvent.empty( ) )
			pAnnounce->setItem( "event", new CAtomString( ann.strEvent ) );

		pAnnounce->setItem( "port", new CAtomLong( ann.iPort ) );
		pAnnounce->setItem( "uploaded", new CAtomLong( ann.iUploaded ) );
		pAnnounce->setItem( "downloaded", new CAtomLong( ann.iDownloaded ) );
		pAnnounce->setItem( "left", new CAtomLong( ann.iLeft ) );
		pAnnounce->setItem( "peer_id", new CAtomString( ann.strPeerID ) );

		struct linkmsg_t lm;

		lm.len = pAnnounce->EncodedLength( );
		lm.type = LINKMSG_ANNOUNCE;
		lm.msg = Encode( pAnnounce );

		if( gpLinkServer )
			gpLinkServer->Queue( lm );
		else if( gpLink )
			gpLink->Queue( lm );

		delete pAnnounce;
	}
	//---------------- End of Addition

	// populate cache

	if( !m_pCached->getItem( ann.strInfoHash ) )
		m_pCached->setItem( ann.strInfoHash, new CAtomList( ) );

	CAtom *pCache = m_pCached->getItem( ann.strInfoHash );

	if( pCache && dynamic_cast<CAtomList *>( pCache ) )
	{
		CAtomList *pCacheList = dynamic_cast<CAtomList *>( pCache );

		if( pCacheList->getValuePtr( )->size( ) < iRSize )
		{
#ifdef BNBT_MYSQL
			if( m_bMySQLOverrideDState )
			{
				CMySQLQuery *pQuery = new CMySQLQuery( "SELECT bid,bip,bport FROM dstate WHERE bhash=\'" + UTIL_StringToMySQL( ann.strInfoHash ) + "\'" );

				vector<string> vecQuery;

				while( ( vecQuery = pQuery->nextRow( ) ).size( ) == 3 )
				{
					CAtomDicti *pAdd = new CAtomDicti( );

					pAdd->setItem( "peer id", new CAtomString( vecQuery[0] ) );
					pAdd->setItem( "ip", new CAtomString( vecQuery[1] ) );
					pAdd->setItem( "port", new CAtomLong( atoi( vecQuery[2].c_str( ) ) ) );

					pCacheList->addItem( pAdd );
				}

				delete pQuery;
			}
			else
			{
#endif
				if( m_pDFile )
				{
					if( !m_pDFile->getItem( ann.strInfoHash ) )
						m_pDFile->setItem( ann.strInfoHash, new CAtomDicti( ) );

					CAtom *pPeers = m_pDFile->getItem( ann.strInfoHash );

					if( pPeers && pPeers->isDicti( ) )
					{
						map<string, CAtom *> *pmapPeersDicti = ( (CAtomDicti *)pPeers )->getValuePtr( );

						for( map<string, CAtom *> :: iterator i = pmapPeersDicti->begin( ); i != pmapPeersDicti->end( ); i++ )
						{
							if( (*i).second->isDicti( ) )
							{
								CAtomDicti *pAdd = new CAtomDicti( );

								CAtom *pIP = ( (CAtomDicti *)(*i).second )->getItem( "ip" );
								CAtom *pPort = ( (CAtomDicti *)(*i).second )->getItem( "port" );

								pAdd->setItem( "peer id", new CAtomString( (*i).first ) );

								if( pIP )
									pAdd->setItem( "ip", new CAtomString( pIP->toString( ) ) );

								if( pPort && dynamic_cast<CAtomLong *>( pPort ) )
									pAdd->setItem( "port", new CAtomLong( *dynamic_cast<CAtomLong *>( pPort ) ) );

								pCacheList->addItem( pAdd );
							}
						}
					}
				}

#ifdef BNBT_MYSQL
			}
#endif

			pCacheList->Randomize( );
		}
	}

	// clamp response

	if( pCache && dynamic_cast<CAtomList *>( pCache ) )
	{
		CAtomList *pCacheList = dynamic_cast<CAtomList *>( pCache );

		CAtom *pPeers = NULL;

		if( strCompact == "1" )
		{
			// compact announce

			string strPeers;

			vector<CAtom *> *pvecList = pCacheList->getValuePtr( );

			for( vector<CAtom *> :: iterator i = pvecList->begin( ); i != pvecList->end( ); )
			{
				if( (*i)->isDicti( ) )
				{
					if( strPeers.size( ) / 6 >= iRSize )
						break;

					CAtom *pIP = ( (CAtomDicti *)(*i) )->getItem( "ip" );
					CAtom *pPort = ( (CAtomDicti *)(*i) )->getItem( "port" );

					if( pIP && pPort && dynamic_cast<CAtomLong *>( pPort ) )
						strPeers += UTIL_Compact( pIP->toString( ), (unsigned int)dynamic_cast<CAtomLong *>( pPort )->getValue( ) );

					delete *i;

					i = pvecList->erase( i );
				}
				else
					i++;
			}

			pPeers = new CAtomString( strPeers );

			// don't compress

			pResponse->bCompressOK = false;
			m_tXStats.announce.iCompact++;
		}
		else
		{
			// regular announce

			CAtomList *pPeersList = new CAtomList( );

			vector<CAtom *> *pvecList = pCacheList->getValuePtr( );

			for( vector<CAtom *> :: iterator i = pvecList->begin( ); i != pvecList->end( ); )
			{
				if( (*i)->isDicti( ) )
				{
					if( pPeersList->getValuePtr( )->size( ) >= iRSize )
						break;

					if( strNoPeerID == "1" )
						( (CAtomDicti *)(*i) )->delItem( "peer id" );

					pPeersList->addItem( new CAtomDicti( *(CAtomDicti *)(*i) ) );

					delete *i;

					i = pvecList->erase( i );
				}
				else
					i++;
			}

			pPeers = pPeersList;

			if( strNoPeerID == "1" )
				m_tXStats.announce.iNopeerid++;
			else
				m_tXStats.announce.iRegular++;
		}

		CAtomDicti *pData = new CAtomDicti( );

		pData->setItem( "interval", new CAtomLong( m_iAnnounceInterval ) );
		pData->setItem( "peers", pPeers );

		// Added by =Xotic=
		CAtom *pFC = m_pFastCache->getItem( strInfoHash );

		if( pFC && dynamic_cast<CAtomList *>( pFC ) )
		{
			vector<CAtom *> vecList = dynamic_cast<CAtomList *>( pFC )->getValue( );

			pData->setItem( "complete", new CAtomInt( *dynamic_cast<CAtomInt *>( vecList[0] ) ) );
			pData->setItem( "incomplete", new CAtomInt( *dynamic_cast<CAtomInt *>( vecList[1] ) ) );
		}
		// ----------------------- End of Addition

		pResponse->strContent = Encode( pData );

		delete pData;
	}
}
