/***
*
* BNBT Beta 8.0 - A C++ BitTorrent Tracker
* Copyright (C) 2003-2004 Trevor Hogan
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
***/

// =Xotic= Modified Source File

#include "bnbt.h"
#include "atom.h"
#include "tracker.h"
#include "util.h"

// Added by =Xotic=
#include "xconfig_lang.h"

void CTracker :: serverResponseComments( struct request_t *pRequest, struct response_t *pResponse )
{
	struct bnbttv btv = UTIL_CurrentTime( );

	if( !m_pAllowed || !m_bAllowComments )
	{
		pResponse->strCode = "403 " + gmapLANG_CFG["server_response_403"];

		return;
	}

	if( pRequest->user.iAccess & ACCESS_VIEW )
	{
		// Output common HTML begin
		HTML_Common_Begin(  pRequest, pResponse, gmapLANG_CFG["comments_page"], string( CSS_COMMENTS ), string( ), NOT_INDEX, CODE_200 );
		JS_Valid_Check( pResponse );

		string strHashString = pRequest->mapParams["info_hash"];
		string strHash = UTIL_StringToHash( strHashString );

		if( !strHash.empty( ) )
		{
			//
			// delete comment
			//

			if( pRequest->user.iAccess & ACCESS_EDIT )
			{
				string strDelAll = pRequest->mapParams["delall"];
				string strDel = pRequest->mapParams["del"];

				if( strDelAll == "1" )
				{
					m_pComments->delItem( strHash );

					saveComments( );

					// Modified by =Xotic=	
					// Deleted all comments.
					pResponse->strContent += "<p class=\"deleted\">" + UTIL_Xsprintf( gmapLANG_CFG["comments_deleted_all"].c_str( ), string( "<a title=\"" + gmapLANG_CFG["comments"] + "\" href=\"/comments.html?info_hash=" + strHashString + "\">" ).c_str( ), "</a>" ) + "</p>\n";

					// Output common HTML ending
					HTML_Common_End( pRequest, pResponse, btv, NOT_INDEX, string( CSS_COMMENTS ) );
					// ---------------------- End of Modification

					return;
				}
				else if( !strDel.empty( ) )
				{
					unsigned int iDel = (unsigned int)( atoi( strDel.c_str( ) ) - 1 );

					CAtom *pComments = m_pComments->getItem( strHash );

					if( pComments && dynamic_cast<CAtomList *>( pComments ) )
					{
						vector<CAtom *> vecComments = dynamic_cast<CAtomList *>( pComments )->getValue( );

						if( iDel < vecComments.size( ) )
						{
							dynamic_cast<CAtomList *>( pComments )->delItem( vecComments[iDel] );

							saveComments( );

							// Modified by =Xotic=	
							// Deleted comment
							pResponse->strContent += "<p class=\"deleted\">" + UTIL_Xsprintf( gmapLANG_CFG["comments_deleted"].c_str( ), strDel.c_str( ), string( "<a title=\"" + gmapLANG_CFG["comments"] + "\" href=\"/comments.html?info_hash=" + strHashString + "\">" ).c_str( ), "</a>" ) + "</p>\n";

							// Output common HTML ending
							HTML_Common_End( pRequest, pResponse, btv, NOT_INDEX, string( CSS_COMMENTS ) );
							// ---------------------- End of Modification

							return;
						}
						else
						{
							// Modified by =Xotic=	
							// Unable to delete comment
							pResponse->strContent += "<p class=\"delete\">" + UTIL_Xsprintf( gmapLANG_CFG["comments_unable_delete"].c_str( ), strDel.c_str( ), string( "<a title=\"" + gmapLANG_CFG["comments"] + "\" href=\"/comments.html?info_hash=" + strHashString + "\">" ).c_str( ), "</a>" ) + "</p>\n";

							// Output common HTML ending
							HTML_Common_End( pRequest, pResponse, btv, NOT_INDEX, string( CSS_COMMENTS ) );
							// ---------------------- End of Modification

							return;
						}
					}
				}
			}

			// display torrent information list

			if( m_pAllowed )
			{
				CAtom *pTorrent = m_pAllowed->getItem( strHash );

				if( pTorrent && dynamic_cast<CAtomList *>( pTorrent ) )
				{
					vector<CAtom *> vecTorrent = dynamic_cast<CAtomList *>( pTorrent )->getValue( );

					if( vecTorrent.size( ) == 6 )
					{
						CAtom *pName = vecTorrent[1];
						CAtom *pAdded = vecTorrent[2];
						CAtom *pSize = vecTorrent[3];
						CAtom *pFiles = vecTorrent[4];
						CAtom *pComment = vecTorrent[5];

						pResponse->strContent += "<div class=\"comments_info_table\">\n";
						pResponse->strContent += "<p>" + gmapLANG_CFG["comments_file_information"] + "</p>\n";
						pResponse->strContent += "<table class=\"comments_info_table\" summary=\"file information\">\n";
						pResponse->strContent += "<tr class=\"comments_info_table\">\n";
						pResponse->strContent += "<td class=\"comments_info_table\">\n";
						pResponse->strContent += "<ul>\n";

						if( pName )
							pResponse->strContent += "<li><strong>" + gmapLANG_CFG["name"] + ":</strong> " + UTIL_RemoveHTML( pName->toString( ) ) + "</li>\n";

						pResponse->strContent += "<li><strong>" + gmapLANG_CFG["info_hash"] + ":</strong> " + strHashString + "</li>\n";

						if( pAdded )
							pResponse->strContent += "<li><strong>" + gmapLANG_CFG["added"] + ":</strong> " + pAdded->toString( ) + "</li>\n";

						if( pSize && dynamic_cast<CAtomLong *>( pSize ) )
							pResponse->strContent += "<li><strong>" + gmapLANG_CFG["size"] + ":</strong> " + UTIL_BytesToString( dynamic_cast<CAtomLong *>( pSize )->getValue( ) ) + "</li>\n";

						if( pFiles && dynamic_cast<CAtomInt *>( pFiles ) )
							pResponse->strContent += "<li><strong>" + gmapLANG_CFG["files"] + ":</strong> " + pFiles->toString( ) + "</li>\n";

						pResponse->strContent += "</ul>\n";
						pResponse->strContent += "</td>\n";
						pResponse->strContent += "</tr>\n";
						pResponse->strContent += "</table>\n";
						pResponse->strContent += "</div>\n";

						if( pComment )
						{
							if( m_bShowFileComment )
							{
								pResponse->strContent += "<div class=\"comments_table\">\n";
								pResponse->strContent += "<p>" + gmapLANG_CFG["file_comment"] + "</p>\n";
								pResponse->strContent += "<table  class=\"comments_table\" summary=\"file comment\">\n";
								pResponse->strContent += "<tr class=\"comments_table\"><td  class=\"comments_table\"><code>" + UTIL_RemoveHTML( pComment->toString( ) ) + "</code></td></tr>\n";
								pResponse->strContent += "</table>\n";
								pResponse->strContent += "</div>\n";
							}
						}
					}
				}
			}

			if( !m_pComments->getItem( strHash ) )
				m_pComments->setItem( strHash, new CAtomList( ) );

			CAtom *pComments = m_pComments->getItem( strHash );

			if( pComments && dynamic_cast<CAtomList *>( pComments ) )
			{
				if( pRequest->user.iAccess & ACCESS_COMMENTS )
				{
					if( pRequest->mapParams.find( "comment" ) != pRequest->mapParams.end( ) )
					{
						string strComment = pRequest->mapParams["comment"].substr( 0, m_iCommentLength );

						if( strComment.empty( ) )
						{
							// Modified by =Xotic=	
							//You must fill in all the fields.
							pResponse->strContent += "<p class=\"fill_all\">" + UTIL_Xsprintf( gmapLANG_CFG["comments_fill_warning"].c_str( ), string( "<a title=\"" + gmapLANG_CFG["comments"] + "\" href=\"/comments.html?info_hash=" + strHashString + "\">" ).c_str( ), "</a>" ) + "</p>\n";

							// Output common HTML ending
							HTML_Common_End( pRequest, pResponse, btv, NOT_INDEX, string( CSS_COMMENTS ) );
							// ---------------------- End of Modification

							return;
						}

						CAtomDicti *pNew = new CAtomDicti( );

						pNew->setItem( "ip", new CAtomString( inet_ntoa( pRequest->sin.sin_addr ) ) );

						if( !pRequest->user.strLogin.empty( ) )
							pNew->setItem( "name", new CAtomString( pRequest->user.strLogin ) );

						pNew->setItem( "comment", new CAtomString( strComment ) );

						time_t tNow = time( NULL );
						char *szTime = asctime( localtime( &tNow ) );
						szTime[strlen( szTime ) - 1] = '\0';

						pNew->setItem( "time", new CAtomString( szTime ) );

						dynamic_cast<CAtomList *>( pComments )->addItem( pNew );

						saveComments( );

						// Modified by =Xotic=	
						// Your comment has been posted.
						pResponse->strContent += "<p class=\"comments_posted\">" + UTIL_Xsprintf( gmapLANG_CFG["comments_posted"].c_str( ), string( "<a title=\"" + gmapLANG_CFG["comments"] + "\" href=\"/comments.html?info_hash=" + strHashString + "\">" ).c_str( ), "</a>" ) + "</p>\n";

						// Output common HTML ending
						HTML_Common_End( pRequest, pResponse, btv, NOT_INDEX, string( CSS_COMMENTS ) );
						// ---------------------- End of Modification

						return;
					}
				}

				vector<CAtom *> *pvecList = dynamic_cast<CAtomList *>( pComments )->getValuePtr( );

				bool bFound = false;

				unsigned long i = 0;

				for( vector<CAtom *> :: iterator it = pvecList->begin( ); it != pvecList->end( ); it++ )
				{
					if( (*it)->isDicti( ) )
					{
						CAtomDicti *pCommentDicti = (CAtomDicti *)(*it);

						CAtom *pIP = pCommentDicti->getItem( "ip" );
						CAtom *pName = pCommentDicti->getItem( "name" );
						CAtom *pComText = pCommentDicti->getItem( "comment" );
						CAtom *pTime = pCommentDicti->getItem( "time" );

						if( pIP && pComText && pTime )
						{
							if( !bFound )
							{
								pResponse->strContent += "<div class=\"comments_table_posted\">";
								pResponse->strContent += "<p class=\"comments_table_posted\">" + gmapLANG_CFG["comments"];

								if( pRequest->user.iAccess & ACCESS_EDIT )
									pResponse->strContent += " [<a title=\"" + gmapLANG_CFG["delete_all"] + "\" href=\"/comments.html?info_hash=" + strHashString + "&amp;delall=1\">" + gmapLANG_CFG["delete_all"] + "</a>]";

								pResponse->strContent += "</p>\n";
								pResponse->strContent += "<table  class=\"comments_table_posted\" summary=\"comments\">\n";

								bFound = true;
							}

							string strIP = pIP->toString( );
							string strName;

							if( pName )
								strName = pName->toString( );

							string strComText = pComText->toString( );
							string strTime = pTime->toString( );

							if( strName.empty( ) )
							{
								// strip ip

								string :: size_type iStart = strIP.rfind( "." );

								if( iStart != string :: npos )
								{
									// don't strip ip for mods

									if( !( pRequest->user.iAccess & ACCESS_EDIT ) )
										strIP = strIP.substr( 0, iStart + 1 ) + "xxx";
								}
							}
							else
							{
								if( !( pRequest->user.iAccess & ACCESS_EDIT ) )
									strIP = gmapLANG_CFG["comment_ip_hidded"];
							}

							//
							// header
							//
				
							// Modified by =Xotic=
							// Comment by
							if( strName.empty( ) )
								strName = gmapLANG_CFG["unknown"];

							if( strTime.empty( ) )
								strTime = gmapLANG_CFG["unknown"];
	
							if( strIP.empty( ) )
								strIP = gmapLANG_CFG["unknown"];

							pResponse->strContent += "<tr class=\"com_header\"><td class=\"com_header\"><code>" + UTIL_Xsprintf( gmapLANG_CFG["comments_posted_by"].c_str( ), CAtomInt( i + 1 ).toString( ).c_str( ) , UTIL_RemoveHTML( strName ).c_str( ) , strIP.c_str( ), strTime.c_str( ) );
							// -------------------------- End of Modification

							if( pRequest->user.iAccess & ACCESS_EDIT )
								pResponse->strContent += " [<a title=\"" + gmapLANG_CFG["delete"] + "\" href=\"/comments.html?info_hash=" + strHashString + "&amp;del=" + CAtomInt( i + 1 ).toString( ) + "\">" + gmapLANG_CFG["delete"] + "</a>]";

							pResponse->strContent += "</code></td></tr>\n";

							//
							// body
							//

							pResponse->strContent += "<tr class=\"com_body\"><td class=\"com_body\"><code>" + UTIL_RemoveHTML( strComText ) + "</code></td></tr>\n";
						}
					}

					i++;
				}

				if( bFound )
				{
					pResponse->strContent += "</table>\n";
					pResponse->strContent += "</div>\n";
				}
				else
					pResponse->strContent += "<p class=\"comments_table_posted\">" + gmapLANG_CFG["comments_no_comment"] + "</p>\n";
			}

			if( pRequest->user.iAccess & ACCESS_COMMENTS )
			{
				pResponse->strContent += "<div class=\"comments_table_post\">\n";
				pResponse->strContent += "<p class=\"comments_table_post\">" + gmapLANG_CFG["comments_post_comment"] + "</p>\n";
				pResponse->strContent += "<table class=\"comments_table_post\">\n";
				pResponse->strContent += "<tr class=\"comments_table_post\">\n";
				pResponse->strContent += "<td class=\"comments_table_post\">\n";
				pResponse->strContent += "<form method=\"get\" action=\"/comments.html\" name=\"postacomment\" onSubmit=\"return validate( this )\">\n";
				pResponse->strContent += "<ul>\n";

				// Comments must be less than
				pResponse->strContent += "<li>" + UTIL_Xsprintf( gmapLANG_CFG["comments_length_info"].c_str( ), CAtomInt( m_iCommentLength ).toString( ).c_str( ) ) + "</li>\n";
				pResponse->strContent += "<li>" + gmapLANG_CFG["no_html"] + "</li>\n";
				pResponse->strContent += "</ul>\n";
				pResponse->strContent += "<p><input name=\"info_hash\" type=hidden value=\"" + strHashString + "\"></p>\n";
				pResponse->strContent += "<p><span style=\"display:none\"><label for=\"commentarea\">" + gmapLANG_CFG["comments_post_comment"] + "</label></span><textarea id=\"commentarea\" name=\"comment\" rows=8 cols=64></textarea></p>\n";
				pResponse->strContent += "<p class=\"comments_table_post\">[<a title=\"" + gmapLANG_CFG["comment_check_length"] + "\" href=\"" + string( JS_CHECKLENGTH ) + "\">" + gmapLANG_CFG["comment_check_length"] + "</a>]</p>\n";
				pResponse->strContent += "<div class=\"comments_table_post_button\">\n";
				Button_Submit( "submit_comment", string( gmapLANG_CFG["submit"] ), pResponse );
				pResponse->strContent += "</div>\n";
				pResponse->strContent += "</form>\n";
				pResponse->strContent += "</td>\n";
				pResponse->strContent += "</tr>\n";
				pResponse->strContent += "</table>\n";
				pResponse->strContent += "</div>\n";
			}
			else
				pResponse->strContent += "<p class=\"denied\">" + gmapLANG_CFG["comments_post_disallowed"] + "</p>\n";
		}
	}
	else
		// Output common HTML begin
		HTML_Common_Begin(  pRequest, pResponse, gmapLANG_CFG["comments_page"], string( CSS_COMMENTS ), string( ), NOT_INDEX, CODE_401 );

	// Modified by =Xotic=	
	// Output common HTML ending
	HTML_Common_End( pRequest, pResponse, btv, NOT_INDEX, string( CSS_COMMENTS ) );
	// ---------------------- End of Modification
}
