/***
*
* XBNBT Beta 81b.3.4 - A C++ BitTorrent Tracker
* Copyright (C) 2003-2004 =Xotic=
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
***/

#ifndef WIN32
#include <dirent.h>
#endif

#include <sys/stat.h>

#include "bnbt.h"
#include "bnbt_mysql.h"
#include "atom.h"
#include "bencode.h"
#include "config.h"
#include "server.h"
#include "tracker.h"
#include "util.h"

// XBNBT Language support
#include "xconfig_lang.h"   

// Initialise XBNBT General variables
void CTracker :: initXGenVars( )
{
	m_strRSSValidImage = CFG_GetString( "xbnbt_rss_valid_image", string( ) );
	m_iUsersOnline = CFG_GetInt( "xbnbt_users_online", 0 );
	m_iNavbar = CFG_GetInt( "xbnbt_show_navbar", 3 );
	m_iShowPeerInfo = CFG_GetInt( "xbnbt_show_peer_info", 0 );
	m_iShowValidator = CFG_GetInt( "xbnbt_show_validator", 0 );
	
	m_iPort = CFG_GetInt( "port", 6969 );
	m_strXTrackerFQDN = CFG_GetString( "xbnbt_tracker_fqdn", string( ) );
	m_strXTrackerURL = "http://" + m_strXTrackerFQDN + ":" + CAtomInt( m_iPort ).toString( ) + "/";
	m_strXDescription = CFG_GetString( "xbnbt_tracker_description", string( ) );
	m_strXTitle = CFG_GetString( "xbnbt_tracker_title", string( ) );
	m_strXKeywords = CFG_GetString( "xbnbt_tracker_keywords", string( ) );

	// The Trinity Edition - Addition Begins
	// Retrieves the "bnbt_use_mouseovers" value
	m_bUseMouseovers = CFG_GetInt( "trinity_use_mouseovers", 0 ) == 0 ? false : true;

	m_bFlagRSSAlert = false;
	m_bFlagXMLAlert = false;
	m_bFlagNotOwnLinkAlert = false;
}

// Initialise XBNBT constant language variables
void CTracker :: initStaticLangVars( )
{
	// Language
	// login1=not logged in
	m_strLogin1 = UTIL_Xsprintf( gmapLANG_CFG["login1"].c_str( ), string( "<a title=\"" + gmapLANG_CFG["login"] + "\" href=\"/login.html\">" ).c_str( ), "</a>" );
	// validate1=validate RSS without images
	m_strValidate1 = UTIL_Xsprintf( "document.write( \"<a rel=\'%s\' title=\'%s\' href=\'http://www.feedvalidator.org/check?url=http://\" + parent.location.host + \"/%s\'>%s<\\/a>\" );", string( TARGET_REL ).c_str( ), gmapLANG_CFG["valid_rss"].c_str( ), rssdump.strName.c_str( ), gmapLANG_CFG["valid_rss"].c_str( ) );
	// validate2=validate RSS with images
	m_strValidate2 = UTIL_Xsprintf( "document.write( \"<a rel=\'%s\' title=\'%s\' href=\'http://\" + parent.location.host + \"/%s\'>http://\" + parent.location.host + \"/%s<img src=\'%s\' alt=\'%s\' height=\'31\' width=\'88\'><\\/a>\" );", string( TARGET_REL ).c_str( ), gmapLANG_CFG["valid_rss"].c_str( ), rssdump.strName.c_str( ), rssdump.strName.c_str( ), m_strRSSValidImage.c_str( ), gmapLANG_CFG["valid_rss"].c_str( ) );
	// JS Reduce Characters - JS_Valid_Check( )
	m_strJSReduce = UTIL_Xsprintf( gmapLANG_CFG["js_reduce_characters"].c_str( ), CAtomInt( m_iCommentLength ).toString( ).c_str( ) );
	// JS Message Length - JS_Valid_Check( )
	m_strJSLength = UTIL_Xsprintf( gmapLANG_CFG["js_message_length"].c_str( ), "\" + theform.comment.value.length + \"" );
	// RSS local link for info.html
	m_strRSSLocalLink = UTIL_Xsprintf( "document.write( \"<a rel=\'%s\' title=\'%s\' href=\'http://\" + parent.location.host + \"/%s\'>http://\" + parent.location.host + \"/%s<\\/a>\" );", string( TARGET_REL ).c_str( ), gmapLANG_CFG["navbar_rss"].c_str( ), rssdump.strName.c_str( ), rssdump.strName.c_str( ) );
}

// Initialise XBNBT tags for internal mouseover
void CTracker :: initTags( )
{
	int iTag = 1;

	string strName = "bnbt_tag" + CAtomInt( iTag ).toString( );
	string strTag;

	// Modified by =Xotic=
	// Internal mouseover
	while( !( strTag = CFG_GetString( strName, string( ) ) ).empty( ) )
	{
		string :: size_type iSplit = strTag.find( "|" );
		string :: size_type iSplitMouse = strTag.find( "&" );

		strName = "bnbt_tag" + CAtomInt( ++iTag ).toString( );

		if( iSplit == string :: npos ) 
		{
			if( iSplitMouse == string :: npos )
			{
				m_vecTags.push_back( pair<string, string>( strTag, string( ) ) );
				m_vecTagsMouse.push_back( pair<string, string>( strTag, string( ) ) );
			}
			else
			{
				m_vecTags.push_back( pair<string, string>( strTag, string( ) ) );
				m_vecTagsMouse.push_back( pair<string, string>( strTag.substr( 0, iSplitMouse ), strTag.substr( iSplitMouse + 1, iSplitMouse + 1) ) );
			}
		}
		else
		{
			if( iSplitMouse == string :: npos )
			{
				m_vecTags.push_back( pair<string, string>( strTag.substr( 0, iSplit ), strTag.substr( iSplit + 1 ) ) );
				m_vecTagsMouse.push_back( pair<string, string>( strTag, string( ) ) );
			}
			else
			{
				m_vecTags.push_back( pair<string, string>( strTag.substr( 0, iSplit ), strTag.substr( iSplit + 1, iSplitMouse - iSplit - 1 ) ) );
				m_vecTagsMouse.push_back( pair<string, string>( strTag.substr( 0, iSplit ), strTag.substr( iSplitMouse +1, iSplitMouse ) ) );
			}
		}   
	}
}

// Initialise XBNBT file server variables
void CTracker :: initServeFileVars( )
{
	// Serve Local
	m_iServeLocal = CFG_GetInt( "xbnbt_serve_local", 1 );
	// robots.txt
	robots.strName = CFG_GetString( "bnbt_robots_txt", string( ) ); 
	// favicon.ico
	favicon.strName = CFG_GetString( "favicon", string( ) );
	// CSS
	style.strName = CFG_GetString( "bnbt_style_sheet", string( ) );
	style.strURL = CFG_GetString( "xbnbt_style_sheet_url", string( ) );
	style.strDir = CFG_GetString( "xbnbt_style_sheet_dir", string( ) );
	// XML
	xmldump.strName = CFG_GetString( "bnbt_dump_xml_file", string( ) );
	m_iDumpXMLInterval = CFG_GetInt( "bnbt_dump_xml_interval", 600 );
	m_iDumpXMLNext = GetTime( ) + m_iDumpXMLInterval;
	// RSS
	rssdump.strName = CFG_GetString( "bnbt_rss_file", string( ) );
	rssdump.strURL = CFG_GetString( "bnbt_rss_online_dir", string( ) );
	rssdump.strDir = CFG_GetString( "xbnbt_rss_directory", string( ) );
	m_strRSSTrackerURL = CFG_GetString( "bnbt_rss_tracker_url", string( ) );
	// image bar
	// fill
	imagefill.strName = CFG_GetString( "image_bar_fill", string( ) );

	if( !imagefill.strName.empty( ) )
	{
	string :: size_type iExt = imagefill.strName.rfind( "." );

	if( iExt != string :: npos )
		imagefill.strExt = imagefill.strName.substr( iExt );
	}
	imagefill.strDir = CFG_GetString( "xbnbt_image_bar_dir", string( ) );
	imagefill.strURL = CFG_GetString( "xbnbt_image_bar_url", string( ) );
	// trans
	imagetrans.strName = CFG_GetString( "image_bar_trans", string( ) );

	if( !imagetrans.strName.empty( ) )
	{
	string :: size_type iExt = imagetrans.strName.rfind( "." );

	if( iExt != string :: npos )
		imagetrans.strExt = imagetrans.strName.substr( iExt );
	}

	imagetrans.strDir = CFG_GetString( "xbnbt_image_bar_dir", string( ) );
	imagetrans.strURL = CFG_GetString( "xbnbt_image_bar_url", string( ) );
}

// XBNBT init vars list
void CTracker :: initXBNBTVarsList( )
{
	// Initialise XBNBT CBTT variables
	initCBTTVars( );

#ifdef XBNBT_GD
	// Initialise XBNBT Dynstat variables
	initDynstatVars( );
#endif

#if defined XBNBT_MYSQL || defined BNBT_MYSQL	
	// Initialise XBNBT MySQL Users variables
	initMySQLUsersVars( );
#endif

	// Initialise XBNBT General variables
	initXGenVars( );

	// XBNBT initialise files served variables
	initServeFileVars( );

	// XBNBT initialise statistics variables
	initXStatsVars( );
	
	// XBNBT initialise Language static variables
	initStaticLangVars( );
	
	// XBNBT initialise tags for internal mouseover
	initTags( );
}

// XBNBT parse list
void CTracker :: initParseList( )
{
	// CBTT parse
	initCBTTParse( );

	// XBNBT Stats
	if( !statsdump.strName.empty( ) )
		importXStats( );

#ifdef XBNBT_GD
	// XBNBT Dynstat
	if( m_iDynstatGenerate && !m_strDynstatFontFile.empty( ) )
		runWriteDynstatImage( );
#endif
	
	// XML
	if( !xmldump.strName.empty( ) )
		saveXML( );
}

// General Buttons
// Submit
void CTracker :: Button_Submit( string sID, string sLabel, struct response_t *pResponse )
{
	pResponse->strContent += "<span style=\"display:none\"><label for=\"" + sID + "\">" + sLabel + "</label></span><input name=\"" + sID + "_button\" id=\"" + sID + "\" alt=\"[" + sLabel + "]\" type=submit value=\"" + sLabel + "\">";
}

// Back
void CTracker :: Button_Back( string sID, string sLabel, struct response_t *pResponse )
{
	pResponse->strContent += "<span style=\"display:none\"><label for=\"" + sID + "\">" + sLabel + "</label></span><input name=\"" + sID + "_button\" id=\"" + sID + "\" alt=\"[" + sLabel + "]\" type=button value=\"" + sLabel + "\" onClick=\"" + string( JS_BACK ) + "\">";
}

// Confirm
void CTracker :: Button_JS_Confirm( string sID, string sLabel, string sJSFunction, struct response_t *pResponse )
{
	pResponse->strContent += "<span style=\"display:none\"><label for=\"" + sID + "\">" + sLabel + "</label></span><input name=\"" + sID + "_button\" id=\"" + sID + "\" alt=\"[" + sLabel + "]\" type=button value=\"" + sLabel + "\" onClick=\"" + sJSFunction + "\">";
}

// Link
void CTracker :: Button_JS_Link( string sID, string sLabel, string sJSFunction, struct response_t *pResponse )
{
	pResponse->strContent += "<span style=\"display:none\"><label for=\"" + sID + "\">" + sLabel + "</label></span><input name=\"" + sID + "_button\" id=\"" + sID + "\" alt=\"[" + sLabel + "]\" type=button value=\"" + sLabel + "\" onClick=\"" + sJSFunction + "\">";
}

void CTracker :: HTML_Nav_Bar( struct request_t *pRequest, struct response_t *pResponse, bool bIndex, string sCSS )
{
	// This funtion builds a navbar
	// start navbar
	pResponse->strContent += "<div class=\"navbar_" + sCSS + "\">\n";
	// Index (RTT)
	if( !bIndex && ( pRequest->user.iAccess & ACCESS_VIEW ) )
	{
		pResponse->strContent += "<span class=\"navbar_index_" + sCSS + "\"><a title=\"" + gmapLANG_CFG["navbar_index"] + "\" class=\"navbar_index_" + sCSS + "\" href=\"/index.html\">" + gmapLANG_CFG["navbar_index"] + "</a></span>\n";
		pResponse->strContent += "<span class=\"navbar_pipe_login_" + sCSS + "\">|</span>";
	}
	// login & my torrents
	if( pRequest->user.strLogin.empty( ) )
		pResponse->strContent += "<span class=\"navbar_login_" + sCSS + "\"><a title=\"" + gmapLANG_CFG["navbar_login"] + "\" class=\"navbar_login_" + sCSS + "\" href=\"/login.html\">" + gmapLANG_CFG["navbar_login"] + "</a></span>\n";
	else
		pResponse->strContent += "<span class=\"navbar_login_" + sCSS + "\"><a title=\"" + gmapLANG_CFG["navbar_my_torrents"] + "\" class=\"navbar_login_" + sCSS + "\" href=\"/login.html\">" + gmapLANG_CFG["navbar_my_torrents"] + "</a></span>\n";
	// upload
	if( pRequest->user.iAccess & ACCESS_UPLOAD )
		pResponse->strContent += "<span class=\"navbar_pipe_upload_" + sCSS + "\">|</span><span class=\"navbar_upload_" + sCSS + "\"><a title=\"" + gmapLANG_CFG["navbar_upload"] + "\" class=\"navbar_upload_" + sCSS + "\" href=\"/upload.html\">" + gmapLANG_CFG["navbar_upload"] + "</a></span>\n";
	// logout
	if( !pRequest->user.strLogin.empty( ) )
		pResponse->strContent += "<span class=\"navbar_pipe_logout_" + sCSS + "\">|</span><span class=\"navbar_logout_" + sCSS + "\"><a title=\"" + gmapLANG_CFG["navbar_logout"] + "\" class=\"navbar_logout_" + sCSS + "\" href=\"/login.html?logout=1\">" + gmapLANG_CFG["navbar_logout"] + "</a></span>\n";
	// signup
	if( pRequest->user.iAccess & ACCESS_SIGNUP )
		pResponse->strContent += "<span class=\"navbar_pipe_signup_" + sCSS + "\">|</span><span class=\"navbar_signup_" + sCSS + "\"><a title=\"" + gmapLANG_CFG["navbar_sign_up"] + "\" class=\"navbar_signup_" + sCSS + "\" href=\"/signup.html\">" + gmapLANG_CFG["navbar_sign_up"] + "</a></span>\n";
	// RSS
	if( ( pRequest->user.iAccess & ACCESS_VIEW ) && !rssdump.strName.empty( ) )
	{
		if( !rssdump.strURL.empty( ) )
			pResponse->strContent += "<span class=\"navbar_pipe_rss_" + sCSS + "\">|</span><span class=\"navbar_rss_" + sCSS + "\"><a rel=\"" + string( TARGET_REL ) + "\" title=\"" + gmapLANG_CFG["navbar_rss"] + "\" class=\"navbar_rss_" + sCSS + "\" href=\"" + rssdump.strURL + rssdump.strName + "\">" + gmapLANG_CFG["navbar_rss"] + "</a></span>\n";
		else if( m_iServeLocal )
			pResponse->strContent += "<span class=\"navbar_pipe_rss_" + sCSS + "\">|</span><span class=\"navbar_rss_" + sCSS + "\"><a rel=\"" + string( TARGET_REL ) + "\" title=\"" + gmapLANG_CFG["navbar_rss"] + "\" class=\"navbar_rss_" + sCSS + "\" href=\"" + rssdump.strName + "\">" + gmapLANG_CFG["navbar_rss"] + "</a></span>\n";
	}
	// XML
	if( ( pRequest->user.iAccess & ACCESS_ADMIN ) && !xmldump.strName.empty( ) )
		pResponse->strContent += "<span class=\"navbar_pipe_xml_" + sCSS + "\">|</span><span class=\"navbar_xml_" + sCSS + "\"><a rel=\"" + string( TARGET_REL ) + "\" title=\"" + gmapLANG_CFG["navbar_xml"] + "\" class=\"navbar_xml_" + sCSS + "\" href=\"" + xmldump.strName + "\">" + gmapLANG_CFG["navbar_xml"] + "</a></span>\n";
	// info
	if( pRequest->user.iAccess & ACCESS_VIEW )
		pResponse->strContent += "<span class=\"navbar_pipe_info_" + sCSS + "\">|</span><span class=\"navbar_info_" + sCSS + "\"><a class=\"navbar_info_" + sCSS + "\" title=\"" + gmapLANG_CFG["navbar_info"] + "\" href=\"/info.html\">" + gmapLANG_CFG["navbar_info"] + "</a></span>\n";
	// admin
	if( pRequest->user.iAccess & ACCESS_ADMIN )
		pResponse->strContent += "<span class=\"navbar_pipe_admin_" + sCSS + "\">|</span><span class=\"navbar_admin_" + sCSS + "\"><a title=\"" + gmapLANG_CFG["navbar_admin"] + "\" class=\"navbar_admin_" + sCSS + "\" href=\"/admin.html\">" + gmapLANG_CFG["navbar_admin"] + "</a></span>\n";
	// users
	if( pRequest->user.iAccess & ACCESS_ADMIN )
		pResponse->strContent += "<span class=\"navbar_pipe_users_" + sCSS + "\">|</span><span class=\"navbar_users_" + sCSS + "\"><a title=\"" + gmapLANG_CFG["navbar_users"] + "\" class=\"navbar_users_" + sCSS + "\" href=\"/users.html\">" + gmapLANG_CFG["navbar_users"] + "</a></span>\n";
	// close navbar
	pResponse->strContent += "</div>\n\n";
}

void CTracker :: HTML_Common_Begin( struct request_t *pRequest, struct response_t *pResponse, string sTitle, string sCSS, string sUrl, bool bIndex, int iCode )
{
	// This function builds the HTML for the start of every display page and is HTML 4.01 Strict compliant
	// Response code and headers
	switch( iCode )
	{
	case CODE_200:
		pResponse->strCode = "200 " + gmapLANG_CFG["server_response_200"];
		pResponse->mapHeaders.insert( pair<string, string>( "Content-Type", string( "text/html; charset=" ) + gstrCharSet ) );
		break;
	case CODE_401:
		pResponse->strCode = "401 " + gmapLANG_CFG["server_response_401"];
		pResponse->mapHeaders.insert( pair<string, string>( "Content-Type", string( "text/html; charset=" ) + gstrCharSet ) );
		pResponse->mapHeaders.insert( pair<string, string>( "WWW-Authenticate", string( "Basic realm=\"" ) + gstrRealm + "\"" ) );
 		pResponse->bCompressOK = false;
		break;
	default:
		;
	}
	// define doctype
	pResponse->strContent += string ( DOC_TYPE );
	// open html and head
	pResponse->strContent += "<html lang=\"en\">\n\n<head>\n";
	// set title
	pResponse->strContent += "<title>";
	if( !m_strXTitle.empty( ) )
	{
		pResponse->strContent += m_strXTitle.c_str( );
		if( !sTitle.empty( ) )
			pResponse->strContent += " | ";
	}
	if( !sTitle.empty( ) )
		pResponse->strContent += sTitle.c_str( );
	pResponse->strContent += "</title>\n";
	// include CSS
	if( !style.strName.empty( ) )
	{
		if( !style.strURL.empty( ) )
			pResponse->strContent += "<link rel=\"stylesheet\" type=\"text/css\" href=\"" + style.strURL + style.strName + "\">\n";
		else
			pResponse->strContent += "<link rel=\"stylesheet\" type=\"text/css\" href=\"" + style.strName + "\">\n";
	}
	// labarks RSS
	if( bIndex && ( pRequest->user.iAccess & ACCESS_VIEW ) && !rssdump.strName.empty( ) )
	{
		if( m_iDumpRSSFileMode == 0 || m_iDumpRSSFileMode == 2 )
		{
			if( !rssdump.strURL.empty( ) )
				pResponse->strContent += "<link rel=\"alternate\" type=\"application/rss+xml\" title=\"RSS\" href=\"" + rssdump.strURL + rssdump.strName + "\">\n";
			else if( m_iServeLocal )
				pResponse->strContent += "<link rel=\"alternate\" type=\"application/rss+xml\" title=\"RSS\" href=\"" + rssdump.strName + "\">\n";
		}
	}
	// test xml
	//if( ( pRequest->user.iAccess & ACCESS_ADMIN ) && !xmldump.strName.empty( ) )
	//	pResponse->strContent += "<link rel=\"alternate\" type=\"application/xml\" title=\"XML\" href=\"" + xmldump.strName + "\">\n";
	// Content-Type
	pResponse->strContent += "<META http-equiv=\"Content-Type\" content=\"text/html; charset=" + gstrCharSet + "\">\n";
	// generator
	pResponse->strContent += "<META name=\"generator\" content=\"" + string( XBNBT_VER ) + "\">\n";
	// Author
	pResponse->strContent += "<META http-equiv=\"Author\" content=\"" + m_strXTitle + "\">\n";
	// Sets index meta tags for search engines and crawlers. All other pages are blocked.
	// You can use robots.txt to control the index page and add further blocking.
	if( bIndex )
	{
		// description meta tag
		if( !m_strXDescription.empty( ) )
			pResponse->strContent += "<META name=\"description\" content=\"" + m_strXDescription + "\">\n";
		// keyword meta tag
		if( !m_strXKeywords.empty( ) )
			pResponse->strContent += "<META name=\"keywords\" content=\"" + m_strXKeywords + "\">\n";
	}
	else
		// robots meta tag
		pResponse->strContent += "<META name=\"robots\" content=\"noindex\">\n";

	// redirect if TRUE
	if( !sUrl.empty( ) )
		pResponse->strContent += "<META http-equiv=\"REFRESH\" content=\"0; url=" + sUrl + "\">\n" ;
	// script-type
	pResponse->strContent += "<META http-equiv=\"Content-Script-Type\" content=\"text/javascript\">\n";
	// close head and start body
	pResponse->strContent += "</head>\n\n<body>\n\n";
	// include JS_Target
	JS_Target( string( TARGET_REL ), string( TARGET_BLANK ), pResponse );
	// include JS_Mouseover_Head if enabled and is index
	if( m_bUseMouseovers && bIndex && ( pRequest->user.iAccess & ACCESS_VIEW ) )
		JS_Mouseover_Head( pResponse );
	// display login status (login1=not logged in) (login2=logged in)
	if( pRequest->user.strLogin.empty( ) )
		pResponse->strContent += "<p class=\"login1_" + sCSS + "\">" + m_strLogin1 + "</p>\n";
	else
		pResponse->strContent += "<p class=\"login2_" + sCSS + "\">" + UTIL_Xsprintf( gmapLANG_CFG["login2"].c_str( ), string( "<span class=\"username\">" + UTIL_RemoveHTML( pRequest->user.strLogin ) + "</span>" ).c_str( ), string( "<a title=\"" + gmapLANG_CFG["navbar_logout"] + "\" href=\"/login.html?logout=1\">" ).c_str( ), "</a>" ) + "</p>\n";
	// display tracker title
	if( bIndex && !m_strXTitle.empty( ) )
        pResponse->strContent += "<h1 class=\"header_" + sCSS + "\">" + m_strXTitle + "</h1>\n\n";
	// display page header
	if( !sTitle.empty( ) )
		pResponse->strContent += "<h3 class=\"header_" + sCSS + "\">" + sTitle + "</h3>\n\n";
	// Display static header
	if( bIndex && ( pRequest->user.iAccess & ACCESS_VIEW ) && !m_strStaticHeader.empty( ) )
		pResponse->strContent += "<div class=\"static_header\">\n" + m_strStaticHeader + "\n</div>\n\n";
	// Display navbar
	if( ( m_iNavbar == 1 ) || ( m_iNavbar == 3 ) )
		HTML_Nav_Bar( pRequest, pResponse, bIndex, sCSS );
	// draw a line
	pResponse->strContent += "<hr class=\"header\">\n\n";
	// display redirect message if TRUE
	if( !sUrl.empty( ) )
		pResponse->strContent += "<p class=\"redirect\">" + UTIL_Xsprintf( gmapLANG_CFG["redirect"].c_str( ), string( "<a title=\"" + sUrl + "\" href=\"" + sUrl + "\">" ).c_str( ), "</a>" ) + "</p>\n\n";
	if( iCode == CODE_401 )
	{
		pResponse->strContent += "<p class=\"unauthorized\">401 " + gmapLANG_CFG["server_response_401"] + "</p>\n" ;
	
		pResponse->strContent += "<p class=\"denied\">" + gmapLANG_CFG["view_not_authorized"];

		if( pRequest->user.iAccess & ACCESS_SIGNUP )
			pResponse->strContent += " " + UTIL_Xsprintf( gmapLANG_CFG["click_to_signup"].c_str( ), string( "<a title=\"" + gmapLANG_CFG["navbar_sign_up"] + "\" href=\"/signup.html\">" ).c_str( ), "</a>" );

		pResponse->strContent += "</p>\n\n";	}
}

void CTracker :: HTML_Common_End( struct request_t *pRequest, struct response_t *pResponse, struct bnbttv sbtv, bool bIndex, string sCSS )
{
	// This function builds the HTML for the end of every display page and is HTML 4.01 Strict compliant
	// Display static footer
	if( bIndex && ( pRequest->user.iAccess & ACCESS_VIEW ) && !m_strStaticFooter.empty( ) )
		pResponse->strContent += "<div class=\"static_footer\">\n" + m_strStaticFooter + "\n</div>\n\n";
	// Draw line
	pResponse->strContent += "<hr class=\"footer\">\n\n";
	// include JS_Mouseover_Tail if enabled and is index
	if( m_bUseMouseovers && bIndex && ( pRequest->user.iAccess & ACCESS_VIEW ) )
		JS_Mouseover_Tail( pResponse );
	// Display navbar
	if( ( m_iNavbar == 2 ) || ( m_iNavbar == 3 ) )
		HTML_Nav_Bar( pRequest, pResponse, bIndex, sCSS );
	// Display users online if enabled and is index
	if( m_iUsersOnline && bIndex && ( pRequest->user.iAccess & ACCESS_VIEW ) )
	{
		int iOnline = ( int )gpServer->m_vecClients.size( );

		if ( iOnline > 1 )
			// Users plural
			pResponse->strContent += "<p class=\"users_online\">" + UTIL_Xsprintf( gmapLANG_CFG["users_online"].c_str( ), CAtomInt( iOnline ).toString( ).c_str( ) ) + "</p>\n\n";
		else
			// User singular
			pResponse->strContent += "<p class=\"users_online\">" + UTIL_Xsprintf( gmapLANG_CFG["user_online"].c_str( ), CAtomInt( iOnline ).toString( ).c_str( ) ) + "</p>\n\n";
	}
	// display the powered_by logo
	if( bIndex )
	{
		// Display Powered By
		pResponse->strContent += "<p class=\"powered_by\">POWERED By \n";
		pResponse->strContent += "<a rel=\"" + string( TARGET_REL ) + "\" title=\"http://sourceforge.net/projects/xbnbt/\" href=\"http://sourceforge.net/projects/xbnbt/\"\n";
		pResponse->strContent += "onMouseOver=\"window.status=\'" + string( XBNBT_VER ) + "\'; return true\"\n"; // BUILD INFORMATION
		pResponse->strContent += "onMouseOut=\"window.status=window.defaultStatus; return true\">" + string( XBNBT_VER ) + "</a> \n";
		pResponse->strContent += "based on BNBT " + string( BNBT_VER ) + "</p>\n\n";
	}
	// Standard page generation time
	if( m_bGen && ( pRequest->user.iAccess & ACCESS_VIEW ) )
		pResponse->strContent  += "<p class=\"gen_" + sCSS + "\">" + UTIL_Xsprintf( gmapLANG_CFG["generated_time"].c_str( ), UTIL_ElapsedTimeStr( sbtv, UTIL_CurrentTime( ) ).c_str( ) ) + "</p>\n\n";
	// Display HTML, CSS & RSS validators
	if( m_iShowValidator || ( pRequest->user.iAccess & ACCESS_ADMIN ) )
	{
		string sCynthia;
		string sHashString = pRequest->mapParams["info_hash"];

		switch( m_iShowValidator )
		{
		case VALID_ADMIN:
		case VALID_TEXT:
			// show validator link only, default for admin
			pResponse->strContent += "<p class=\"html_valid_" + sCSS + "\">\n";
			// HTML
			pResponse->strContent += "<a rel=\"" + string( TARGET_REL ) + "\" title=\"" + gmapLANG_CFG["valid_html"] + "\" href=\"http://validator.w3.org/check?uri=referer\">" + gmapLANG_CFG["valid_html"] + "</a>\n";
			// CSS
			pResponse->strContent += "<span class=\"pipe\">|</span><a rel=\"" + string( TARGET_REL ) + "\" title=\"" + gmapLANG_CFG["valid_css"] + "\" href=\"http://jigsaw.w3.org/css-validator/check/referer\">" + gmapLANG_CFG["valid_css"] + "</a>\n";
			// RSS
			if( !rssdump.strName.empty( ) )
			{
				if( !rssdump.strURL.empty( ) )
					pResponse->strContent += "<span class=\"pipe\">|</span><a rel=\"" + string( TARGET_REL ) + "\" class=\"rss_valid\" title=\"" + gmapLANG_CFG["valid_rss"] + "\" href=\"http://www.feedvalidator.org/check?url=" + rssdump.strURL + rssdump.strName + "\">" + gmapLANG_CFG["valid_rss"] + "</a>\n";
				else if( m_iServeLocal )
					pResponse->strContent += "<span class=\"pipe\">|</span><script type=\"text/javascript\">" + m_strValidate1 + "</script>";
			}
			// Cynthia
			if( sHashString.empty( ) )
				sCynthia = UTIL_Xsprintf( "document.write( \"<a rel=\'%s\' title=\'%s\' href=\'http://www.contentquality.com/mynewtester/cynthia.exe?rptmode=2&url1=http://\" + parent.location.host + \"%s\'>%s<\\/a>\" );", string( TARGET_REL ).c_str( ), gmapLANG_CFG["cynthia_tested"].c_str( ), string( pRequest->strURL ).c_str( ), gmapLANG_CFG["cynthia_tested"].c_str( ) );
			else
				sCynthia = UTIL_Xsprintf( "document.write( \"<a rel=\'%s\' title=\'%s\' href=\'http://www.contentquality.com/fulloptions.asp?rptmode=2&url1=http://\" + parent.location.host + \"%s?info_hash=%s\'>%s<\\/a>\" );", string( TARGET_REL ).c_str( ), gmapLANG_CFG["cynthia_tested"].c_str( ), string( pRequest->strURL ).c_str( ), sHashString.c_str( ), gmapLANG_CFG["cynthia_tested"].c_str( ) );

			pResponse->strContent += "<span class=\"pipe\">|</span><script type=\"text/javascript\">" + sCynthia + "</script>";
			// Close
			pResponse->strContent += "</p>\n\n";
			break;
		case VALID_IMAGE:
			// show validator with image
			pResponse->strContent += "<p class=\"html_valid_" + sCSS + "\">\n";
			// HTML
			pResponse->strContent += "<a rel=\"" + string( TARGET_REL ) + "\" title=\"" + gmapLANG_CFG["valid_html"] + "\" href=\"http://validator.w3.org/check?uri=referer\"><img style=\"border:0;width:88px;height:31px\" src=\"http://www.w3.org/Icons/valid-html401\" alt=\"" + gmapLANG_CFG["valid_html"] + "\"></a>\n";
			// CSS
			pResponse->strContent += "<span class=\"pipe\">|</span><a rel=\"" + string( TARGET_REL ) + "\" title=\"" + gmapLANG_CFG["valid_css"] + "\" href=\"http://jigsaw.w3.org/css-validator/check/referer\"><img style=\"border:0;width:88px;height:31px\" src=\"http://jigsaw.w3.org/css-validator/images/vcss\" alt=\"" + gmapLANG_CFG["valid_css"] + "\"></a>\n";
			// RSS
			if( !rssdump.strName.empty( ) )
			{
				if( !rssdump.strURL.empty( ) )
					pResponse->strContent += "<span class=\"pipe\">|</span><a rel=\"" + string( TARGET_REL ) + "\" class=\"rss_valid\" title=\"" + gmapLANG_CFG["valid_rss"] + "\" href=\"http://www.feedvalidator.org/check?url=" + rssdump.strURL + rssdump.strName + "\"><img style=\"border:0;width:88px;height:31px\" src=\"" + m_strRSSValidImage + "\" alt=\"" + gmapLANG_CFG["valid_rss"] + "\"</a>\n";
				else if( m_iServeLocal )
					pResponse->strContent += "<span class=\"pipe\">|</span><script type=\"text/javascript\">" + m_strValidate2 + "</script>";
			}
			// Cynthia
			if( sHashString.empty( ) )
				sCynthia = UTIL_Xsprintf( "document.write( \"<a rel=\'%s\' title=\'%s\' href=\'http://www.contentquality.com/mynewtester/cynthia.exe?rptmode=2&url1=http://\" + parent.location.host + \"%s\'><img style=\"border:0;width:88px;height:31px\" src=\"http://www.CynthiaSays.com/images/Ctested.gif\" alt=\"%s\"><\\/a>\" );", string( TARGET_REL ).c_str( ), gmapLANG_CFG["cynthia_tested"].c_str( ), string( pRequest->strURL ).c_str( ), gmapLANG_CFG["cynthia_tested"].c_str( ) );
			else
				sCynthia = UTIL_Xsprintf( "document.write( \"<a rel=\'%s\' title=\'%s\' href=\'http://www.contentquality.com/fulloptions.asp?rptmode=2&url1=http://\" + parent.location.host + \"%s?info_hash=%s\'><img style=\"border:0;width:88px;height:31px\" src=\"http://www.CynthiaSays.com/images/Ctested.gif\" alt=\"%s\"><\\/a>\" );", string( TARGET_REL ).c_str( ), gmapLANG_CFG["cynthia_tested"].c_str( ), string( pRequest->strURL ).c_str( ), sHashString.c_str( ), gmapLANG_CFG["cynthia_tested"].c_str( ) );

			pResponse->strContent += "<span class=\"pipe\">|</span><script type=\"text/javascript\">" + sCynthia + "</script>";
			// Close
			pResponse->strContent += "</p>\n\n";
			break;
		default:
			;
		}
	}

	// Final responses
	// XML Reset Alert
	if( m_bFlagXMLAlert )
	{
		pResponse->strContent += "<script type=\"text/javascript\">alert(\"XML Updated\")</script>\n\n";
		m_bFlagXMLAlert = false;
	}

	// RSS Reset Alert
	if( m_bFlagRSSAlert )
	{
		pResponse->strContent += "<script type=\"text/javascript\">alert(\"RSS Updated\")</script>\n\n";
		m_bFlagRSSAlert = false;
	}

	// RSS Reset Alert
	if( m_bFlagNotOwnLinkAlert )
	{
		pResponse->strContent += "<script type=\"text/javascript\">alert(\"" + gmapLANG_CFG["admin_not_owner"] + "\")</script>\n\n";
		m_bFlagNotOwnLinkAlert = false;
	}

	// Close body and and html
	pResponse->strContent += "</body>\n</html>\n";
}   

//  Mutex for link
void CTracker :: QueueAnnounce( struct announce_t ann )
{
	// normally called from link.cpp
	m_mtxQueued.Claim( );
	m_vecQueued.push_back( ann );
	m_mtxQueued.Release( );
} 

// XBNBT Responses
void CTracker :: serverResponseXML( struct response_t *pResponse )
{
	pResponse->strCode = "200 " + gmapLANG_CFG["server_response_200"];

	pResponse->mapHeaders.insert( pair<string, string>( "Content-Type", gmapMime[".xml"] ) );

	pResponse->strContent = xmldump.strFile;
}

void CTracker :: serverResponseRSS( struct response_t *pResponse )
{
	pResponse->strCode = "200 " + gmapLANG_CFG["server_response_200"];

	pResponse->mapHeaders.insert( pair<string, string>( "Content-Type", gmapMime[".rss"] ) );

	pResponse->strContent = rssdump.strFile;
}

void CTracker :: serverResponseCSS( struct response_t *pResponse )
{
	pResponse->strCode = "200 " + gmapLANG_CFG["server_response_200"];

	pResponse->mapHeaders.insert( pair<string, string>( "Content-Type", gmapMime[".css"] ) );

	pResponse->strContent = style.strFile;
}

void CTracker :: serverResponseImagefill( struct response_t *pResponse )
{
	pResponse->strCode = "200 " + gmapLANG_CFG["server_response_200"];

	pResponse->mapHeaders.insert( pair<string, string>( "Content-Type", gmapMime[imagefill.strExt] ) );

	pResponse->strContent = imagefill.strFile;
}

void CTracker :: serverResponseImagetrans( struct response_t *pResponse )
{
	pResponse->strCode = "200 " + gmapLANG_CFG["server_response_200"];

	pResponse->mapHeaders.insert( pair<string, string>( "Content-Type", gmapMime[imagetrans.strExt] ) );

	pResponse->strContent = imagetrans.strFile;
}

// Set Link Established Date and Time
void CTracker :: setLinkDate( )
{
	m_tXStats.date.sLinkEstablished = gstrLinkEstDate;
}

// Destroy XBNBT
void CTracker :: destroyXBNBT( )
{
	saveRSS( );
	saveXML( );
	exportXStats( );
	saveDFile( );

	delete m_pDynstat;
	delete m_pXStats;
	// XBNBT MySQL Users Integration
#if defined XBNBT_MYSQL || defined BNBT_MYSQL
	if( giMySQLUsersOverrideUsers )
		if( m_pMySQLUsersUsers )
			delete m_pMySQLUsersUsers;
#endif

	m_pDynstat = NULL;
	m_pXStats = NULL;
	// XBNBT MySQL Users Integration
#if defined XBNBT_MYSQL || defined BNBT_MYSQL
	if( giMySQLUsersOverrideUsers )
		m_pMySQLUsersUsers = NULL;   
#endif
}
