/***
*
* XBNBT Beta 81b.3.4 - A C++ BitTorrent Tracker
* Copyright (C) 2003-2004 =Xotic=
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
***/

#include "bnbt.h"
#include "atom.h"
#include "tracker.h"
#include "util.h"

// XBNBT Language support
#include "xconfig_lang.h"   

void CTracker :: JS_Noscript( struct response_t *pResponse )
{
	// Display a message if javascript not supported by browser
	pResponse->strContent += "<noscript>\n";    
	pResponse->strContent += "<p class=\"js_warning\">" + gmapLANG_CFG["js_enable_support"] + "</p>\n";   
	pResponse->strContent += "</noscript>\n\n"; 
}

// The main Javascript
void CTracker :: JS_Target( string sRel, string sTarget, struct response_t *pResponse )
{
	// Display a message if javascript not supported by browser
	JS_Noscript( pResponse );
	// This function facilitates the replacement of target="_blank" with an equivalent
	// in javascript fot HTML 4.01 Strict compliance
	// Usage:
	// where you used <a target="_blank" ....
	// now use <a rel="external" ...
	// then when called, the java script will intepret rel="external" as requires new browser
	pResponse->strContent += "<script type=\"text/javascript\">\n\n";
	pResponse->strContent += "<!--\n";
	pResponse->strContent += "function externalLinks() {\n";
	pResponse->strContent += "  if (!document.getElementsByTagName) return;\n";
	pResponse->strContent += "      var anchors = document.getElementsByTagName(\"a\");\n";
	pResponse->strContent += "  for (var i=0; i<anchors.length; i++) {\n";
	pResponse->strContent += "      var anchor = anchors[i];\n";
	pResponse->strContent += "      if (anchor.getAttribute(\"href\") && anchor.getAttribute(\"rel\") == \"" + sRel + "\")\n";
	pResponse->strContent += "      anchor.target = \"" + sTarget + "\";\n";
	pResponse->strContent += "  }\n";
	pResponse->strContent += "}\nwindow.onload = externalLinks;\n";
	pResponse->strContent += "//-->\n\n";
	pResponse->strContent += "</script>\n\n";   
}

// For internalised mouseover feature
void CTracker :: JS_Mouseover_Head( struct response_t *pResponse )
{
	int iImage = 1;

	pResponse->strContent += "<script type=\"text/javascript\">\n\n";
	pResponse->strContent += "<!--\n";

	if( !m_vecTagsMouse.empty( ) )
	{
		for( vector< pair< string, string > > :: iterator i = m_vecTagsMouse.begin( ); i != m_vecTagsMouse.end( ); i++ )
		{
			pResponse->strContent += "imagetag" + CAtomInt( iImage ).toString( ) + " = new Image();\n";
			pResponse->strContent += "imagetag" + CAtomInt( iImage ).toString( ) + ".src = \"" + (*i).second + "\";\n";
			pResponse->strContent += "function hoverOff" + UTIL_StringToEscaped( (*i).first ) + "() {\n";
			pResponse->strContent += " document." + UTIL_StringToEscaped( (*i).first ) + ".src = \"" + (*i).second + "\";\n";
			pResponse->strContent += "}\n\n";

			iImage++;
		}
	}

	if( !m_vecTags.empty( ) )
	{       
		for( vector< pair< string, string > > :: iterator i = m_vecTags.begin( ); i != m_vecTags.end( ); i++ )
		{
			pResponse->strContent += "imagetag" + CAtomInt( iImage ).toString( ) + " = new Image();\n";
			pResponse->strContent += "imagetag" + CAtomInt( iImage ).toString( ) + ".src = \"" + (*i).second + "\";\n";
			pResponse->strContent += "function hoverOn" + UTIL_StringToEscaped( (*i).first ) + "() {\n";
			pResponse->strContent += "document." + UTIL_StringToEscaped( (*i).first ) + ".src = \"" + (*i).second + "\";\n";
			pResponse->strContent += "}\n\n";

			iImage++;
		}   
	}
	pResponse->strContent += "//-->\n";
	pResponse->strContent += "</script>\n\n";
}

// For internalised mouseover feature
void CTracker :: JS_Mouseover_Tail( struct response_t *pResponse )
{
	pResponse->strContent += "<script type=\"text/javascript\">\n";
	pResponse->strContent += "<!--\n";

	for( vector< pair< string, string > > :: iterator i = m_vecTagsMouse.begin( ); i != m_vecTagsMouse.end( ); i++ )
	{
		pResponse->strContent += "document." + UTIL_StringToEscaped( (*i).first ) + ".src = \"" + (*i).second + "\";\n";
	}

	pResponse->strContent += "//-->\n";
	pResponse->strContent += "</script>\n\n";
}

// Javascript for admin
void CTracker :: JS_Admin( struct response_t *pResponse )
{
	pResponse->strContent += "<script type=\"text/javascript\">\n";
	pResponse->strContent += "<!--\n";

	pResponse->strContent += "function admin_kill() {\n";
	pResponse->strContent += "window.location=\"/admin.html?ap_kill=1\"\n";
	pResponse->strContent += "}\n\n";

	pResponse->strContent += "function admin_count() {\n";
	pResponse->strContent += "window.location=\"/admin.html?ap_recount=1\"\n";
	pResponse->strContent += "}\n\n";

	pResponse->strContent += "function admin_relink() {\n";
	pResponse->strContent += "window.location=\"/admin.html?ap_relink=1\"\n";
	pResponse->strContent += "}\n\n";

	pResponse->strContent += "function admin_rss() {\n";
	pResponse->strContent += "window.location=\"/admin.html?ap_rss=1\"\n";
	pResponse->strContent += "}\n\n";

	pResponse->strContent += "function admin_xml() {\n";
	pResponse->strContent += "window.location=\"/admin.html?ap_xml=1\"\n";
	pResponse->strContent += "}\n\n";

	pResponse->strContent += "function admin_dynstat() {\n";
	pResponse->strContent += "window.location=\"/admin.html?ap_dynstat=1\"\n";
	pResponse->strContent += "}\n\n";

	pResponse->strContent += "function admin_announce() {\n";
	pResponse->strContent += "window.location=\"/admin.html?ap_announce=1\"\n";
	pResponse->strContent += "}\n\n";

	pResponse->strContent += "function admin_scrape() {\n";
	pResponse->strContent += "window.location=\"/admin.html?ap_scrape=1\"\n";
	pResponse->strContent += "}\n\n";

	pResponse->strContent += "function admin_file() {\n";
	pResponse->strContent += "window.location=\"/admin.html?ap_file=1\"\n";
	pResponse->strContent += "}\n\n";

	pResponse->strContent += "function admin_page() {\n";
	pResponse->strContent += "window.location=\"/admin.html?ap_page=1\"\n";
	pResponse->strContent += "}\n\n";

	pResponse->strContent += "function admin_all() {\n";
	pResponse->strContent += "window.location=\"/admin.html?ap_all=1\"\n";
	pResponse->strContent += "}\n\n";

	pResponse->strContent += "function admin_rtp() {\n";
	pResponse->strContent += "window.location=\"/admin.html\"\n";
	pResponse->strContent += "}\n\n";

	pResponse->strContent += "function kill_confirm()\n";
	pResponse->strContent += "{\n";
	pResponse->strContent += "var name=confirm(\"" + gmapLANG_CFG["js_kill_the_tracker_q"] + "\")\n";
	pResponse->strContent += "if (name==true)\n";
	pResponse->strContent += "{\n";
	pResponse->strContent += "admin_kill()\n";
	pResponse->strContent += "}\n";
	pResponse->strContent += "}\n\n";

	pResponse->strContent += "function relink_confirm()\n";
	pResponse->strContent += "{\n";
	pResponse->strContent += "var name=confirm(\"" + gmapLANG_CFG["js_reset_tracker_link_q"] + "\")\n";
	pResponse->strContent += "if (name==true)\n";
	pResponse->strContent += "{\n";
	pResponse->strContent += "admin_relink()\n";
	pResponse->strContent += "}\n";
	pResponse->strContent += "}\n\n";

	pResponse->strContent += "function dynstat_confirm()\n";
	pResponse->strContent += "{\n";
	pResponse->strContent += "var name=confirm(\"" + gmapLANG_CFG["js_reset_dynstat_stats_q"] + "\")\n";
	pResponse->strContent += "if (name==true)\n";
	pResponse->strContent += "{\n";
	pResponse->strContent += "admin_dynstat()\n";
	pResponse->strContent += "}\n";
	pResponse->strContent += "}\n\n";

	pResponse->strContent += "function announce_confirm()\n";
	pResponse->strContent += "{\n";
	pResponse->strContent += "var name=confirm(\"" + gmapLANG_CFG["js_reset_announce_stats_q"] + "\")\n";
	pResponse->strContent += "if (name==true)\n";
	pResponse->strContent += "{\n";
	pResponse->strContent += "admin_announce()\n";
	pResponse->strContent += "}\n";
	pResponse->strContent += "}\n\n";

	pResponse->strContent += "function scrape_confirm()\n";
	pResponse->strContent += "{\n";
	pResponse->strContent += "var name=confirm(\"" + gmapLANG_CFG["js_reset_scrape_stats_q"] + "\")\n";
	pResponse->strContent += "if (name==true)\n";
	pResponse->strContent += "{\n";
	pResponse->strContent += "admin_scrape()\n";
	pResponse->strContent += "}\n";
	pResponse->strContent += "}\n\n";

	pResponse->strContent += "function file_confirm()\n";
	pResponse->strContent += "{\n";
	pResponse->strContent += "var name=confirm(\"" + gmapLANG_CFG["js_reset_file_stats_q"] + "\")\n";
	pResponse->strContent += "if (name==true)\n";
	pResponse->strContent += "{\n";
	pResponse->strContent += "admin_file()\n";
	pResponse->strContent += "}\n";
	pResponse->strContent += "}\n\n";

	pResponse->strContent += "function page_confirm()\n";
	pResponse->strContent += "{\n";
	pResponse->strContent += "var name=confirm(\"" + gmapLANG_CFG["js_reset_page_stats_q"] + "\")\n";
	pResponse->strContent += "if (name==true)\n";
	pResponse->strContent += "{\n";
	pResponse->strContent += "admin_page()\n";
	pResponse->strContent += "}\n";
	pResponse->strContent += "}\n\n";

	pResponse->strContent += "function all_confirm()\n";
	pResponse->strContent += "{\n";
	pResponse->strContent += "var name=confirm(\"" + gmapLANG_CFG["js_reset_all_stats_q"] + "\")\n";
	pResponse->strContent += "if (name==true)\n";
	pResponse->strContent += "{\n";
	pResponse->strContent += "admin_all()\n";
	pResponse->strContent += "}\n";
	pResponse->strContent += "}\n\n";

	pResponse->strContent += "//-->\n";
	pResponse->strContent += "</script>\n\n";
}

// Javascript for Return to admin
void CTracker :: JS_ReturnToAdmin( struct response_t *pResponse )
{
	pResponse->strCode = "200 " + gmapLANG_CFG["server_response_200"];
	pResponse->mapHeaders.insert( pair<string, string>( "Content-Type", string( "text/html; charset=" ) + gstrCharSet ) );
	// define doctype
	pResponse->strContent += string ( DOC_TYPE );
	// open html and head,close head and start body
	pResponse->strContent += "<html lang=\"en\">\n\n<head>\n</head>\n\n<body>\n\n";
	// Display a message if javascript not supported by browser
	JS_Noscript( pResponse );
	// Run Script
	pResponse->strContent += "<script type=\"text/javascript\">window.location=\"/admin.html\"</script>\n\n";
	// Close body and html
	pResponse->strContent += "</body>\n</html>\n\n";
}

// Javascript for comments
void CTracker :: JS_Valid_Check( struct response_t *pResponse )
{
	// assorted scripts (thanks SA)
	pResponse->strContent += "<script type=\"text/javascript\">\n";
	pResponse->strContent += "<!--\n";

	// validate
	pResponse->strContent += "function validate( theform ) {\n";
	pResponse->strContent += "if( theform.comment.value == \"\" ) {\n";
	pResponse->strContent += "  alert( \"" + gmapLANG_CFG["js_fill_fields"] + "\" );\n";
	pResponse->strContent += "  return false; }\n";
	pResponse->strContent += "if( theform.comment.value.length > " + CAtomInt( m_iCommentLength ).toString( ) + " ) {\n";
	pResponse->strContent += "  alert( \"" + gmapLANG_CFG["js_message_too_long"] + "\\n" + m_strJSReduce + "\\n" + m_strJSLength + "\" );\n";
	pResponse->strContent += "  return false; }\n";
	pResponse->strContent += "else { return true; }\n";
	pResponse->strContent += "}\n\n";

	// checklength
	pResponse->strContent += "function checklength( theform ) {\n";
	pResponse->strContent += "  alert( \"" + m_strJSLength + "\" );\n";
	pResponse->strContent += "}\n";
	pResponse->strContent += "//-->\n";
	pResponse->strContent += "</script>\n\n";
}

